<?php


namespace App\Services\pay;

use Api\PhpServices\Idgen\Idgen;
use App\Models\Dictionary;
use App\Models\order\mysql\PayOrder;
use App\Models\order\mysql\PayOrderItem;
use App\Models\order\mysql\PayOrderClearing;
use App\Models\order\mysql\PayOrderClearingItem;
use App\Services\Order\OrderService;

use App\Services\pingxx\PingxxService;
use App\Services\wallet\WalletService;
use Helpers\Strategy;
use Helpers\Logger;

use App\Exception\custom\PayException;

class PayService
{
    // 订单信息
    private $order_info = [];
    // 子订单信息
    private $order_items = [];
    // 分销信息
    private $order_distribution = [];

    private $pay_order = [];
    private $pay_order_items = [];
    private $clear_list = [];
    private $clear_items_list = [];


    public function do_pay($order_id, $user_id) {

        // 获取订单信息 + 子单信息 + 分润信息
        $data = OrderService::getOrderData($order_id, $user_id);
        if(empty($data['order'])) {
            throw new PayException(['cus' => 1]);
        }

        $order = $data['order'];
        // 判断是否存在有效订单
        $expire_time = time() + 30 * 60;
        $pay_order_id = $this->gen_pay_order_id($order_id);
        $pay_order = [
            'pay_order_id'=>$pay_order_id,
            'user_id'=>$order['user_id'],
            'life_account_id'=>$order['life_account_id'],
            'shop_id'=>$order['shop_id'],
            'order_id'=>$order['order_id'],
            'pay_order_status'=>Dictionary::O_PAY_STATUS_UNPAY,
            'pay_amount'=>$order['payment'],
            'third_order_id'=>'',
            'expire_time'=>$expire_time,
            'is_distribution'=>$order['is_distribution'] ?? 0,
            'source_name'=>1,
            'service_name'=>1,
            'extra'=> json_encode([]),
        ];

        //获取或创建支付订单
        $order = PayOrder::get_valid_order($order_id, $pay_order);
        if(empty($order)) {
            throw new PayException(['cus' => 0]);
        }

        if($order['pay_order_status'] == Dictionary::O_PAY_STATUS_PAYED) {
            throw new PayException(['cus' => 3]);
        }

        // 支持幂等，存在有效订单需要可以重复支付
        $ret = PingxxService::createOrder($pay_order);
        if($ret["status"] == 'paid') {
            $edit = [
                'pay_order_status'=>Dictionary::O_PAY_STATUS_PAYED,
                'third_order_id'=>$ret['id'],
                //'pay_channel'=>$ret['charge_essentials']['channel'],
            ];
            PayOrder::update($edit, ['pay_order_id'=>$order['pay_order_id']]);

        } elseif($ret["status"] == 'created') {
            $edit = [
                'pay_order_status'=>Dictionary::O_PAY_STATUS_WAIT,
                'third_order_id'=>$ret['id'],
            ];
            PayOrder::update($edit, ['pay_order_id'=>$order['pay_order_id']]);

        } elseif($ret["status"] == 'canceled') {
            $edit = [
                'expire_time'=>date('Y-m-d H:i:s'),
            ];
            PayOrder::update($edit, ['pay_order_id'=>$order['pay_order_id']]);

        }else{}

        return $ret;
    }

    /**
     * @throws PayException
     * 回调订单支付
     */
    public function call_back($data) {

        if(empty($data['data']['object'])) {
            Logger::error('回调参数错误：', $data);
            throw new PayException(['cus' => 5]);
        }

        $object = $data['data']['object'];

        $ping_data = PingxxService::getOrder($object['id']);
        if($ping_data['status'] != 'paid') {
            throw new PayException(['cus' => 5]);
        } //actual_amount 金额核对

        // 修改状态，锁定业务
        $pay_order_id = $ping_data['merchant_order_no'];

        $this->lock_pay_order($pay_order_id);

        //获取订单 + 子单 + 分销信息
        $ret = OrderService::getOrderData($this->pay_order['order_id'], $this->pay_order['user_id']);
        if(empty($ret['order'])) {
            throw new PayException(['cus' => 1]);
        }

        // 生成支付清分单
        $this->make_order_clearing($ret);

        $this->make_order_clear();
    }

    public function write_off() {
        //锁定状态
    }

    /**
     * @param $pay_order_id
     * @throws PayException
     * 锁定支付状  态
     */
    private function lock_pay_order($pay_order_id) {
        try{

            PayOrder::beginTransaction();
            $pay_order = PayOrder::getMaster('*', ['pay_order_id'=>$pay_order_id]);
            if(empty($pay_order)) {
                throw new PayException(['cus' => 0]);
            }

            $this->pay_order = $pay_order;

            // 如果已经支付，返回成功保持幂等。
            if($pay_order['pay_order_status'] == Dictionary::O_PAY_STATUS_PAYED) {
                throw new RefundException(['cus' => 0]);
            }

            $edit = [
                'pay_order_status'=>Dictionary::O_PAY_STATUS_PAYED,
                'pingxx_callback_success_time'=>date('Y-m-d H:i:s'),
            ];

            PayOrder::update($edit, ['pay_order_id'=>$pay_order_id]);
            PayOrder::commit();

        }catch (Exception $e) {
            PayOrder::rollback();
            Logger::error($e->getMessage(), $edit);
            throw $e;
        }
    }

    /**
     * @param $pay_order_id
     * 处理清分
     */
    private function make_order_clear() {

        $this->wallet_list = [];

        $data = OrderService::getOrderData($this->pay_order['order_id'], $this->pay_order['user_id']);
        //平台收入、团长、商户
        foreach ($data['order_item'] as $item) {
            if($item['marketing_type'] == 1) {
                $this->make_order_item_clear($item);
            }
        }

        if($this->wallet_list) {
            $ret = WalletService::send($this->send_wallet());
            PayOrderItem::update([
                'notify_account_status'=>2,
                'notify_account_success_time'=>date('Y-m-d H:i:s')
            ], ['pay_order_id'=>$this->pay_order['order_id'], 'notify_account_status'=>1]);
        }


    }

    /**
     * @param $order_item
     * 结算单个子单
     */
    private function make_order_item_clear($order_item) {

        if(empty($this->pay_order)) {
            throw new PayException(['cus' => 0]);
        }

        PayOrderItem::update([
            'notify_account_status'=>1,
        ], ['order_item_id'=>$order_item['order_item_id'], 'notify_account_status'=>0]);

        $clearing_items = PayOrderClearingItem::select('*',
            ['order_item_id'=>$order_item['order_item_id'], 'pay_order_id'=>$this->pay_order['pay_order_id']]
        );

        foreach ($clearing_items as $row) {
            $this->wallet_list[] = [
                'account_id'=>$row['account_id'],
                'pay_amount'=>$row['pay_amount'],
                'pay_type'=>$row['pay_type'],
                'trade_id'=>$row['pay_order_clearing_item_id'],
            ];
        }
    }



    /**
     * @param $order_id
     * @param $user_id
     * @throws PayException
     * 判断是否有效订单，如果存在则调用三方判断订单状态
     */
    private function exist_valid($order_id, $order) {


    }

    /**
     * @param $data
     * 构造订单流水
     */
    private function make_order_clearing($data) {

        $this->order_info = $data['order'];
        $this->order_items = $data['order_items'];
        $this->order_distribution = $data['order_distribution'];

        $this->clear_list = [];
        $this->clear_items_list = [];
        $this->pay_order_items = [];

        foreach ($this->order_items as $r) {
            $this->pay_order_items[] = [
                'pay_order_item_id' => $this->gen_pay_order_item_id(),
                'pay_order_id' => $this->order_info['pay_order_id'],
                'order_id' => $this->order_info['order_id'],
                'order_item_id' => $r['order_item_id'],
                'user_id' => $this->order_info['user_id'],
            ];
        }

        $wx_tip = $this->make_tip_clearing(); //微信手续费
        $subsidy_tip = $this->make_platform_subsidy_clearing(); //系统补贴
        $merchant_tip = $this->make_merchant_subsidy_clearing(); //商家补贴

        $distribution_tip = $this->make_distribution_clearing(); //团长收益
        $platform_tip = $this->make_platform_clearing(); //平台收益

        $merchant_cash = $this->order_info['payment'] + $subsidy_tip + $merchant_tip - $wx_tip - $distribution_tip - $platform_tip;
        $this->make_merchant_clearing($merchant_cash); //商户收益

        try{
            PayOrderClearing::beginTransaction();
            $i_cnt = PayOrderClearing::insert($this->pay_order_items, ['rowCount'=>true]);
            $c_cnt = PayOrderItem::insert($this->clear_list, ['rowCount'=>true]);
            $ci_cnt = PayOrderClearingItem::insert($this->clear_items_list, ['rowCount'=>true]);

            if($i_cnt * $c_cnt * $ci_cnt == 0) {
                throw new PayException(['cus'=>0]);
            }

            PayOrderClearing::commit();
        }catch (\Exception $e) {
            Logger::error($e->getMessage(), []);
            PayOrderClearing::rollback();
            throw new PayException(['cut'=>5]);
        }
    }

    /**
     * @param $payment
     * @return false|float
     * 获取微信手续费
     */
    private function make_tip_clearing() {
        $total_tip = round(0.006 * $this->order_info['payment']);
        $account = [
            'account_id'=>'100020003001',
            'amount'=>$total_tip,
            'title'=>'微信手续费',
        ];

        if(empty($total_tip)) {
            return 0;
        }

        $account['account_type'] = Dictionary::ACCOUNT_TYPE_ST;
        $account['pay_type'] = Dictionary::PAY_TYPE_IN;
        $account['pay_sub_type'] = Dictionary::PAY_SUB_TYPE_WX;
        $account['need_recorded'] = Dictionary::NO;

        /* 微信手续费总单，在退款、核销的时候会更新这个金额，保证他的准确 */
        $this->do_clearing_data($account, $total_tip, true);
        return $total_tip;
    }

    /**
     * 计算平台运营补贴
     */
    private function make_platform_subsidy_clearing() {

        return 0; //本期不做平台补贴
        $account = $this->get_marketing_subsidy();
        $total_tip = $account['amount'];
        if(empty($total_tip)) {
            return 0;
        }

        $account['account_type'] = Dictionary::ACCOUNT_TYPE_SS;
        $account['pay_type'] = Dictionary::PAY_TYPE_OUT;
        $account['pay_sub_type'] = Dictionary::PAY_SUB_TYPE_SYS_OUT;
        $account['need_recorded'] = Dictionary::NO;

        $this->do_clearing_data($account, $total_tip, false);
        return $total_tip;
    }

    /**
     * 计算商户运营补贴
     */
    private function make_merchant_subsidy_clearing() {
        return 0; // 本期不做商户补贴
        $account = $this->get_marketing_subsidy();
        $total_tip = $account['amount'];
        if(empty($total_tip)) {
            return 0;
        }

        $account['account_type'] = Dictionary::ACCOUNT_TYPE_M;
        $account['pay_type'] = Dictionary::PAY_TYPE_OUT;
        $account['pay_sub_type'] = Dictionary::PAY_SUB_TYPE_M_OUT;
        $account['need_recorded'] = Dictionary::NO;

        $this->do_clearing_data($account, $total_tip, false);
        return $total_tip;
    }

    /**
     * @param $total
     * @return false|float
     * 计算平台收入
     */
    private function make_platform_clearing() {
        $account = [
            'account_id'=>'100020003201',
            'amount'=>0,
            'rate'=>60,
            'title'=>'平台收益',
        ];

        $total_tip = $account['amount'] = round($this->order_info['total_price'] * $account['rate'] / 10000);
        if(empty($total_tip)) {
            return 0;
        }

        $account['account_type'] = Dictionary::ACCOUNT_TYPE_SS;
        $account['pay_type'] = Dictionary::PAY_TYPE_IN;
        $account['pay_sub_type'] = Dictionary::PAY_SUB_TYPE_SYS_IN;
        $account['need_recorded'] = Dictionary::NO;

        $this->do_clearing_data($account, $total_tip, false);
        return $total_tip;
    }

    /**
     * @return mixed
     * 计算分销信息
     */
    private function make_distribution_clearing() {

        if($this->order_info['is_distribution'] == 0) {
            return 0;
        }

        /**
        'distributor_user_id'=>122234,
        'distributor_commission_value'=>60,
        'parent_user_id'=>122234 . rand(111, 999),
        'parent_commission_value'=>60,
         */

        $list = [
            [
                'user_id'=>$this->order_distribution['distributor_user_id'],
                'commission_value'=>$this->order_distribution['distributor_commission_value'],
                'commission_type'=>$this->order_distribution['distributor_commission_type'],
            ],
            [
                'user_id'=>$this->order_distribution['parent_user_id'],
                'commission_value'=>$this->order_distribution['parent_commission_value'],
                'commission_type'=>$this->order_distribution['parent_commission_type'],
            ],
        ];

        $total_tip = 0;

        foreach ($list as $account) {

            if($account['commission_type'] == 1) {//rate
                $account['amount'] = round($account['commission_value'] * $this->order_info['total_price'] / 10000);
            } else { //定值
                $account['amount'] = $account['commission_value'];
            }

            $account['account_id'] = $account['account_id'];
            $account['account_type'] = Dictionary::ACCOUNT_TYPE_M;
            $account['pay_type'] = Dictionary::PAY_TYPE_OUT;
            $account['pay_sub_type'] = Dictionary::PAY_SUB_TYPE_M_OUT;
            $account['need_recorded'] = Dictionary::NO;

            $this->do_clearing_data($account, $account['amount'], false);

            $total_tip += $account['amount'];
        }

        return $total_tip;
    }

    /**
     * @param $merchant_cash
     * @return mixed
     * 计算商户清分信息
     */
    private function make_merchant_clearing($merchant_cash) {

        $user_id = $this->get_life_account($this->order_info['life_account_id']);
        $account = [
            'account_id'=>$user_id,
            'amount'=>$merchant_cash,
            'title'=>'商家订单收益',
        ];

        $account['account_type'] = Dictionary::ACCOUNT_TYPE_M;
        $account['pay_type'] = Dictionary::PAY_TYPE_IN;
        $account['pay_sub_type'] = Dictionary::PAY_SUB_TYPE_M_IN;
        $account['need_recorded'] = Dictionary::NO;

        $this->do_clearing_data($account, $merchant_cash, false);
        return $merchant_cash;

    }

    private function do_clearing_data($account, $total_tip, $is_wx = false) {

        $total_amount = $this->order_info['payment']; //订单支付总价格
        $cleared_amount = 0; //已经结算记录
        $cleared_tip = 0; //已经结算记录

        $pay_order_clearing_id = $this->gen_pay_order_clearing_id();
        $this->clear_list[] = [
            'pay_order_clearing_id' => $pay_order_clearing_id,
            'pay_order_id' => $this->pay_order['pay_order_id'],
            'order_id' => $this->order_info['order_id'],
            'account_id' => $account['account_id'],
            'account_type' => $account['account_type'],
            'pay_amount' => $account['amount'],
            'pay_type' => $account['pay_type'],
            'pay_sub_type' => $account['pay_sub_type'],
            'need_recorded' => $account['need_recorded'],
        ];

        if($is_wx) { //微信不计算手续费拆弹
            return true;
        }

        foreach ($this->order_items as $r) {

            $current_tip = Strategy::getTip($r['payment'], $total_amount, $cleared_amount, $total_tip, $cleared_tip);

            $this->clear_items_list[] = [
                'pay_order_clearing_item_id' => $this->gen_pay_order_clearing_item_id(),
                'pay_order_clearing_id' => $pay_order_clearing_id,
                'pay_order_id' => $this->order_info['pay_order_id'],
                'order_id' => $this->order_info['order_id'],
                'account_id' => $account['account_id'],
                'account_type' => $account['account_type'],
                'pay_amount' => $current_tip,
                'pay_type' => $account['pay_type'],
                'pay_sub_type' => $account['pay_sub_type'],
                'need_recorded' => $account['need_recorded'],
            ];

            $cleared_amount += $r['payment'];
            $cleared_tip += $current_tip;
        }
    }


    /**
     * @param $order_id
     * @param $user_id
     * @return string[]
     * 获取支付订单的营销补贴，返回补贴出款账号，补贴金额，补贴原因
     */
    private function get_marketing_subsidy() {
        return [
            'account_id'=>'100020003001',
            'amount'=>1000,
            'title'=>'营销专号支付补贴',
        ];
    }

    /**
     * @param $order
     * @return string[]
     * 获取订单优惠券补贴，一般来自订单，需要这里完善出款账号 + 原因
     */
    private function get_coupon_subsidy($order) {
        return [
            'account_id'=>'100020002001',
            'amount'=>'0',
            'title'=>'营销专号支付补贴',
        ];
    }

    private function gen_pay_order_id($user_id) {
        $number = substr($user_id, -2);
        return $this->get_idgen_id($number, 1);
    }

    private function gen_pay_order_item_id($user_id) {
        $number = substr($user_id, -2);
        return $this->get_idgen_id($number, 1);
    }

    private function gen_pay_order_clearing_id() {
        $number = substr($this->order_info['user_id'], -2);
        return $this->get_idgen_id($number, 1);
    }

    private function gen_pay_order_clearing_item_id() {
        $number = substr($this->order_info['user_id'], -2);
        return $this->get_idgen_id($number, 1);
    }

    private function get_idgen_id($number, $count=1){
        $res = Idgen::get(appConfig('idgen.partner'),appConfig('idgen.key'), [], [[
            "type" => "55c768",
            'number'=>$number,
            "count"=> $count]]);
        $id = $res['id_datetime']['55c768'] ?? [];
        return $id;
    }

    private function get_life_account($life_account_id) {

        $url = config('interface', 'merchant.lifeaccount.get_life_account_by_id');
        if (!$url) {
            throw new CodeSpecialException("failed");
        }

        $params = ['life_account_id'=>$life_account_id];
        return Sdk::call($url, $params);
    }

    /**
     * @param $data
     * 构造订单支付订单信息
     */
    private function make_new_pay($data) {
        $this->make_new_pay_order($data['order']);
    }
}