<?php


namespace App\Services\pingxx;

use Pingpp\Order;
use Pingpp\OrderRefund;

use Helpers\Logger;


class PingxxService
{

    /**
     * @param $order_item_id
     * @param $user_id
     * @throws RefundException
     * 发起退款请求
     */
    public static function sendRefund($refundData, $chargeId) {

        try {
            Logger::info('退款订单发起', ['chargeId'=>$chargeId, 'refundData'=>$refundData]);
            $royalty_users = [];

            $ret = OrderRefund::create(
                $refundData['pay_order_id'],
                [
                    'charge' => $chargeId,  //在该订单中，需要退款的 charge 对象 ID，默认全部退款。
                    'charge_amount' => $refundData['refund_amount'],    //charge 退款金额，默认为全额退款。必须和 charge 参数同时使用。单位：分。
                    'description' => '用户申请退款',    //退款附加说明。
                    'metadata' => [
                        'refund_order_id'=>$refundData['refund_order_id'],
                        'order_item_id'=>$refundData['order_item_id'],
                        'goods_sku_id'=>$refundData['goods_sku_id'] ?? ''
                    ],
                    'refund_mode' => 'to_source',              //退款模式。原路退回：to_source，退至余额：to_balance。默认为原路返回。
                    'royalty_users' => $royalty_users,
                ]
            );

            Logger::info('退款订单返回', ['res'=>$ret]);
            $respose = json_decode($ret, true);
            return $respose['data'];

            /*
             {
              "object": "list",
              "url": "/v1/orders/2111608270000005209/order_refunds",
              "has_more": false,
              "data": [
                {
                  "id": "re_y1u944PmfnrTHyvnL0nD0iD1",
                  "object": "refund",
                  "order_no": "y1u944PmfnrTHyvnL0nD0iD1",
                  "amount": 800,
                  "created": 1499930518,
                  "succeed": true,
                  "status": "succeeded",
                  "time_succeed": 1499930518,
                  "description": "Refund Description",
                  "failure_code": null,
                  "failure_msg": null,
                  "metadata": {},
                  "charge": "ch_8SCSCCn90ir1bb54m5fjbnX5",
                  "charge_order_no": "2017071102122327",
                  "transaction_no": "2004450349201512090096425284",
                  "extra": {}
                }
              ]
            }
             *
             */

        } catch (\Pingpp\Error\Base $e) {
            if ($e->getHttpStatus() != null) {
                header('Status: ' . $e->getHttpStatus());
                echo $e->getHttpBody();
            } else {
                echo $e->getMessage();
            }
        }
    }

    /**
     * @param $orderId ping++ 支付订单ID
     * @param $refundId ping++ 退款ID
     * @return mixed
     * 退款信息查询
     */
    public static function getRefund($orderId, $refundId) {
        try {
            Logger::info('退款查询发起', ['orderId'=>$orderId, 'refundId'=>$refundId]);
            $ret = OrderRefund::retrieve($orderId, $refundId);
            Logger::info('退款查询返回', ['res'=>$ret]);
            $respose = json_decode($ret, true);
            return $respose;

        } catch (\Pingpp\Error\Base $e) {
            if ($e->getHttpStatus() != null) {
                header('Status: ' . $e->getHttpStatus());
                echo $e->getHttpBody();
            } else {
                echo $e->getMessage();
            }
        }
    }

    /**
     * @param $order
     * @return mixed
     * 创建订单
     */
    public static function createOrder($order) {
        try {
            Logger::info('创建订单发起', $order);
            $royalty_users = [];
            $ret = Order::create(
                [
                    "amount" => $order['pay_amount'],
                    "app" => APP_ID,
                    "merchant_order_no" => $order['pay_order_id'],
                    "subject" => "用户购买",
                    "currency" => "cny",
                    "body" => "购买商品",
                    "uid" => $order['user_id'],
                    "client_ip" => "192.168.0.101",
                    'receipt_app' => APP_ID,    // 收款方应用
                    'service_app' => APP_ID,    // 服务方应用
                    'royalty_users' => $royalty_users,
                    //'balance_settlement' => [   // 可选参数，余额结算信息。
                    //    'user' => 'user_001',
                    //    'user_fee' => 1
                    //]
                ]
            );

            Logger::info('创建订单返回', ['res'=>$ret]);
            $respose = json_decode($ret, true);
            return $respose;

        } catch (\Pingpp\Error\Base $e) {
            // 捕获报错信息
            if ($e->getHttpStatus() != null) {
                echo $e->getHttpStatus() . PHP_EOL;
                echo $e->getHttpBody() . PHP_EOL;
            } else {
                echo $e->getMessage() . PHP_EOL;
            }
        }
    }

    /**
     * @param $orderId
     * 订单查询
     */
    public static function getOrder($orderId) {

        try {
            Logger::info('查询订单发起', ['orderId'=>$orderId]);
            $ret = Order::retrieve($orderId);

            Logger::info('查询订单返回', ['res'=>$ret]);
            $respose = json_decode($ret, true);
            return $respose;

        } catch (\Pingpp\Error\Base $e) {
            if ($e->getHttpStatus() != null) {
                header('Status: ' . $e->getHttpStatus());
                echo $e->getHttpBody();
            } else {
                echo $e->getMessage();
            }
        }
    }

    /**
     * @param $raw
     * @param $header
     * @return int
     * 校验回调是否合法
     */
    public static function verifySignature($raw, $headers) {

        $signature = isset($headers['X-Pingplusplus-Signature']) ? $headers['X-Pingplusplus-Signature'] : null;

        $pub_key_path = '';
        $pub_key_contents = file_get_contents($pub_key_path);
        return openssl_verify($raw, base64_decode($signature), $pub_key_contents, 'sha256');
    }
}